using System;
using System.IO;
using System.Drawing;
using System.Windows.Forms;

// Need this to get the path to the images
using System.Reflection;

namespace AlphaExample
{
    public partial class MainForm : Form
    {
        protected Bitmap backBuffer;
        protected Image constantAlphaImage;
        protected IImage imagingImage;
        protected IImage imagingResource;

        public MainForm()
        {
            InitializeComponent();

            backBuffer = new Bitmap(this.ClientSize.Width, this.ClientSize.Height, System.Drawing.Imaging.PixelFormat.Format32bppRgb);

            // The other radio buttons have their tag set in InitializeComponent().
            radioButton4.Tag = Color.FromArgb(255, 255, 192);

            // Load the image to use with the AlphaBlend API.
            string path = System.IO.Path.GetDirectoryName(Assembly.GetExecutingAssembly().GetName().CodeBase); 
            constantAlphaImage = new Bitmap(path + @"\blendme.bmp");

            // Load the image with alpha data through Imaging.
            // We don't need the factory later, so it is local (and temporary).
            IImagingFactory factory = (IImagingFactory)Activator.CreateInstance(Type.GetTypeFromCLSID(new Guid("327ABDA8-072B-11D3-9D7B-0000F81EF32E")));
            factory.CreateImageFromFile(path + @"\ihavealpha.png", out imagingImage);

            // Load the image with alpha data from an embedded resource through Imaging
            // !! If your data source is not a MemoryStream, you will need to get your image data into a byte array and use it below. !!
            MemoryStream strm = (MemoryStream)Assembly.GetExecutingAssembly().GetManifestResourceStream("AlphaExample.embedded.png");
            byte[] pbBuf = strm.GetBuffer();
            uint cbBuf = (uint)strm.Length;
            factory.CreateImageFromBuffer(pbBuf, cbBuf, BufferDisposalFlag.BufferDisposalFlagNone, out imagingResource);
        }

        protected override void OnPaintBackground(PaintEventArgs e)
        {
            // Prevent flicker, we will take care of the background in Form1_Paint()
        }

        private void Form1_Paint(object sender, PaintEventArgs e)
        {
            // We are going to double buffer to cut down on flicker.
            // This isn't foolproof on slower devices (we can see the fullscreen Blt).
            // You could trade space for speed and keep this background buffer around
            // rather than (re)creating it each time Paint is called.  I would hook Resize and
            // create a new back buffer in that event.

            // The bitmap needs to be created with the 32bpp pixel format for the IImage to do the right thing.
            if (backBuffer != null)
            {
                // We beed a Graphics object on the buffer to get an HDC
                using (Graphics gxBuffer = Graphics.FromImage(backBuffer))
                {
                    // Since we nop'd OnPaintBackground, take care of it here
                    gxBuffer.Clear(this.BackColor);

                    // Ask the Image object from Imaging to draw itself.
                    IntPtr hdcDest = gxBuffer.GetHdc();
                    Rectangle dstRect = new Rectangle(100, 100, 148, 148);
                    imagingResource.Draw(hdcDest, ref dstRect, IntPtr.Zero);
                    gxBuffer.ReleaseHdc(hdcDest);

                    // AlphaBlend takes two HDC's - one source and one destination.  Here's the source.
                    using (Graphics gxSrc = Graphics.FromImage(constantAlphaImage))
                    {
                        IntPtr hdcDst = gxBuffer.GetHdc();
                        IntPtr hdcSrc = gxSrc.GetHdc();
                        BlendFunction blendFunction = new BlendFunction();
                        blendFunction.BlendOp = (byte)BlendOperation.AC_SRC_OVER;   // Only supported blend operation
                        blendFunction.BlendFlags = (byte)BlendFlags.Zero;           // Documentation says put 0 here
                        blendFunction.SourceConstantAlpha = (byte)vScrollBar1.Value;// Constant alpha factor
                        blendFunction.AlphaFormat = (byte)0;                        // Don't look for per pixel alpha
                        PlatformAPIs.AlphaBlend(hdcDst, 36, 36, 128, 128, hdcSrc, 0, 0, 128, 128, blendFunction);
                        gxBuffer.ReleaseHdc(hdcDst);    // Required cleanup to GetHdc()
                        gxSrc.ReleaseHdc(hdcSrc);       // Required cleanup to GetHdc()
                    }

                    // Ask the Image object from Imaging to draw itself.
                    /*IntPtr*/ hdcDest = gxBuffer.GetHdc(); // This is redundant, but keeps the necessary code together
                    /*Rectangle*/ dstRect = new Rectangle(4, 4, 132, 132);
                    imagingImage.Draw(hdcDest, ref dstRect, IntPtr.Zero);
                    gxBuffer.ReleaseHdc(hdcDest);
                }

                // Put the final composed image on screen.
                e.Graphics.DrawImage(backBuffer, 0, 0);
            }
            else
                e.Graphics.Clear(this.BackColor);
        }

        protected override void OnResize(EventArgs e)
        {
            if (backBuffer != null)
                backBuffer.Dispose();
            backBuffer = new Bitmap(this.ClientSize.Width, this.ClientSize.Height, System.Drawing.Imaging.PixelFormat.Format32bppRgb);
            base.OnResize(e);
        }

        private void vScrollBar1_ValueChanged(object sender, EventArgs e)
        {
            // Prompt a repaint using the new value of the scrollbar for the constantAlphaImage alpha
            this.Refresh();
        }

        private void radioButton_CheckedChanged(object sender, EventArgs e)
        {
            if (((RadioButton)sender).Checked)
            {
                // Update the form BackColor and prompt a repaint
                this.BackColor = (Color)((RadioButton)sender).Tag;
                this.Refresh();
            }
        }
    }
}